unit u_list;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Buttons, Menus, ImgList, ComCtrls;

type
  TdlgList = class(TForm)
    ImageList1: TImageList;
    Panel3: TPanel;
    SpeedButton2: TBitBtn;
    BitBtn1: TBitBtn;
    BitBtn3: TBitBtn;
    PopupMenu1: TPopupMenu;
    Delete1: TMenuItem;
    PopupMenu2: TPopupMenu;
    MenuItem1: TMenuItem;
    Panel1: TPanel;
    PageControl1: TPageControl;
    tsDone: TTabSheet;
    lvDone: TListView;
    tsCopy: TTabSheet;
    lvCopy: TListView;
    tsDel: TTabSheet;
    lvDel: TListView;
    procedure FormCreate(Sender: TObject);
    procedure lvDoneColumnClick(Sender: TObject; Column: TListColumn);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure Delete1Click(Sender: TObject);
    procedure MenuItem1Click(Sender: TObject);
    procedure PopupMenu2Popup(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private-Deklarationen }
    procedure ParseLists;
    procedure DelSel(lv: TListView);
    procedure SaveList(const fName: string);
  public
    { Public-Deklarationen }
  end;

var
  dlgList: TdlgList;

implementation

uses u_main, u_str;

{$R *.DFM}

var
  listCounts: array[0..2, 0..1] of Int64 = ((0, 0), (0, 0), (0, 0));

procedure TdlgList.ParseLists;

  procedure AddItem(I: TFileRec; ls: Pointer; const aIx: LongInt);
  var
    lv: TListView;
    impl: Integer;
  begin
    with i do begin
      if IsDone = adToDo then
      begin
        if Reason in [arDelFile, arDelDir]
          then
          lv := lvDel
        else
          lv := lvCopy;
      end
      else
        lv := lvDone;
      if IsDone = adFailed
        then
        impl := 0
      else
        impl := -1;

      Inc(listCounts[lv.Tag, 0]);
      Inc(listCounts[lv.Tag, 1], size_src);
      with lv.Items.Add do
      begin
        Data := ls;
        Caption := ExtractFileName(DelBackSlash(name_dst));
        if lv = lvDone then
          case IsDone of
            adToDo: SubItems.Add(rsIDToDo);
            adSuccess: SubItems.Add(rsIDSuccess);
            adFailed: SubItems.Add(rsIDFailed);
          end;
        SubItems.Add(DelBackSlash(ExtractFilePath(DelBackSlash(name_src))));
        if not (Reason in [arDelFile, arDelDir])
          then
          SubItems.Add(DelBackSlash(ExtractFilePath(DelBackSlash(name_dst))))
        else
          if lv = lvDone
            then
            SubItems.Add('');
        case Reason of
          arNewFile:
            begin
              SubItems.Add(rslnf);
              ImageIndex := 0;
              OverlayIndex := impl;
            end;
          arNewDir:
            begin
              SubItems.Add(rslnd);
              ImageIndex := 1;
              OverlayIndex := impl;
            end;
          arDelFile:
            begin
              SubItems.Add(rsldf);
              ImageIndex := 2;
              OverlayIndex := impl;
            end;
          arDelDir:
            begin
              SubItems.Add(rsldd);
              ImageIndex := 3;
              OverlayIndex := impl;
            end;
          arAttr:
            begin
              SubItems.Add(Format(rslsa, [DiffAttrToStr(DiffAttr)]));
              ImageIndex := 4;
              OverlayIndex := impl;
            end;
          arSize:
            begin
              SubItems.Add(Format(rslds, [Readable(size_src), Readable(size_dst)]));
              ImageIndex := 5;
              OverlayIndex := impl;
            end;
          arDate:
            begin
              SubItems.Add(Format(rsldt, [ReadableFileTime(date_src), ReadableFileTime(date_dst)]));
              ImageIndex := 5;
              OverlayIndex := impl;
            end;
        end;
        SubItems.AddObject(Readable(size_src), Pointer(size_src + 1));
        SubItems.AddObject('', Pointer(aIX));
      end;
    end;
  end;


var
  ct: Integer;
begin
  Screen.Cursor := crHourGlass;
  listCounts[0, 0] := 0;
  listCounts[1, 0] := 0;
  listCounts[2, 0] := 0;
  listCounts[0, 1] := 0;
  listCounts[1, 1] := 0;
  listCounts[2, 1] := 0;
  lvDone.Items.BeginUpdate;
  lvCopy.Items.BeginUpdate;
  lvDel.Items.BeginUpdate;
  try
    lvDone.ViewStyle := vsList;
    lvCopy.ViewStyle := vsList;
    lvDel.ViewStyle := vsList;
    lvDone.Items.Clear;
    lvCopy.Items.Clear;
    lvDel.Items.Clear;
    lvDone.AllocBy := CopyList.Count + DelList.Count;
    lvCopy.AllocBy := CopyList.Count + DelList.Count;
    lvDel.AllocBy := CopyList.Count + DelList.Count;
    if CopyList.Count > 0
      then
      for ct := 0 to Pred(CopyList.Count)
        do
        if Assigned(CopyList[ct])
          then
          AddItem(PFileRec(CopyList[ct])^, CopyList, ct);

    if DelList.Count > 0
      then
      for ct := 0 to Pred(DelList.Count)
        do
        if Assigned(DelList[ct])
          then
          AddItem(PFileRec(DelList[ct])^, DelList, ct);

    tsDel.TabVisible := lvDel.Items.Count > 0;
    tsCopy.TabVisible := lvCopy.Items.Count > 0;
    tsDone.TabVisible := (lvDone.Items.Count > 0) or ((not tsDel.TabVisible) and (not tsCopy.TabVisible));
    tsDone.Caption := Format(rslap, [Readable(listCounts[0, 0]), Readable(listCounts[0, 1])]);
    tsCopy.Caption := Format(rsllc, [Readable(listCounts[1, 0]), Readable(listCounts[1, 1])]);
    tsDel.Caption := Format(rslld, [Readable(listCounts[2, 0]), Readable(listCounts[2, 1])]);
  finally
    lvDone.ViewStyle := vsReport;
    lvCopy.ViewStyle := vsReport;
    lvDel.ViewStyle := vsReport;
    Screen.Cursor := crDefault;
    lvDone.Items.EndUpdate;
    lvCopy.Items.EndUpdate;
    lvDel.Items.EndUpdate;
  end;
end;

procedure TdlgList.FormCreate(Sender: TObject);
begin
  ImageList1.Overlay(6, 0);
  ParseLists;

end;

function csortproc(lParam1, lParam2, lParamSort: Integer): Integer stdcall;
var
  s1, s2: string;
  n1, n2: Int64;
begin
  if lParamSort = 0 then
  begin
    s1 := TListItem(lParam1).Caption;
    s2 := TListItem(lParam2).Caption;
    n1 := 0;
    n2 := 0;
  end
  else
  begin
    s1 := TListItem(lParam1).SubItems[lParamSort - 1];
    s2 := TListItem(lParam2).SubItems[lParamSort - 1];
    n1 := Int64(TListItem(lParam1).SubItems.Objects[lParamSort - 1]);
    n2 := Int64(TListItem(lParam2).SubItems.Objects[lParamSort - 1]);
  end;
  if (n1 * n2) <> 0
    then
    Result := n1 - n2
  else
    Result := lstrcmpi(PChar(s1), PChar(s2)); //StrToInt(s1) - StrToIntDef(s2,0);
end;


procedure TdlgList.lvDoneColumnClick(Sender: TObject; Column: TListColumn);
begin
  TListView(Sender).CustomSort(@csortproc, Column.Index);
end;

procedure TdlgList.DelSel(lv: TListView);
var
  ct: Integer;
  li: TList;
  ix: LongInt;
begin
  if (lv.SelCount > 0) and (MessageDlg(rslcd, mtConfirmation, [mbyes, mbno], 0) = idyes) then
  try
    Screen.Cursor := crHourGlass;
    for ct := 0 to Pred(lv.Items.Count)
      do
      if lv.Items[ct].Selected then
      begin
        li := lv.Items[ct].Data;
        ix := Integer(lv.Items[ct].SubItems.Objects[Pred(lv.Items[ct].SubItems.Count)]);
        if Assigned(li) and (li.Count > ix) and Assigned(li[ix]) then
        begin
          FreeMem(PFileRec(li[ix]));
          li[ix] := nil;
        end;
      end;
    CopyList.Pack;
    DelList.Pack;
    ParseLists;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TdlgList.PopupMenu1Popup(Sender: TObject);
begin
  Delete1.Enabled := Bool(lvCopy.SelCount);
end;

procedure TdlgList.Delete1Click(Sender: TObject);
begin
  DelSel(lvCopy);
end;

procedure TdlgList.MenuItem1Click(Sender: TObject);
begin
  DelSel(lvDel);
end;

procedure TdlgList.PopupMenu2Popup(Sender: TObject);
begin
  MenuItem1.Enabled := Bool(lvDel.SelCount);
end;

procedure TdlgList.SpeedButton2Click(Sender: TObject);
begin
  with fmMain.SaveDialog1 do
  try
    Filter := rslsf;
    Title := rslst;
    Options := Options - [ofNoDereferenceLinks];
    DefaultExt := 'TXT';
    FileName := '';
    if Execute
      then
      SaveList(FileName);
  finally
  end;
end;

procedure TdlgList.SaveList(const fName: string);
var
  sr: tstrings;
  i: Integer;
begin
  sr := tstringlist.create;
  try
    sr.add(defcap);
    sr.add(Format(rslda, [datetimetostr(now)]));
    if tsDone.TabVisible then
    begin
      sr.Add('== ' + tsDone.Caption);
      for i := 0 to Pred(lvDone.Items.Count)
        do
        with lvDone.Items[i] do
        begin
          sr.Add(Format('%s: %s\%s => %s\%s: (%s %s), status %s', [Subitems[3], DelBackSlash(SubItems[1]),
            Caption, DelBackSlash(SubItems[2]), Caption, rslss, SubItems[4], SubItems[0]]));
        end;
      sr.Add('');
    end;
    if tsCopy.TabVisible then
    begin
      sr.Add('== ' + tsCopy.Caption);
      for i := 0 to Pred(lvCopy.Items.Count)
        do
        with lvCopy.Items[i] do
        begin
          sr.Add(Format('%s : %s\%s => %s\%s : (%s %s)', [Subitems[2], DelBackSlash(SubItems[0]),
            Caption, DelBackSlash(SubItems[1]), Caption, rslss, SubItems[3]]));
        end;
      sr.Add('');
    end;
    if tsDel.TabVisible then
    begin
      sr.Add('== ' + tsDel.Caption);
      for i := 0 to Pred(lvDel.Items.Count)
        do
        with lvDel.Items[i] do
        begin
          sr.Add(Format('%s : %s\%s : (%s %s)', [Subitems[1], DelBackSlash(SubItems[0]),
            Caption, rslss, SubItems[2]]));
        end;
      sr.Add('');
    end;
    sr.savetofile(fname);
  finally
    sr.free;
  end;
end;

procedure TdlgList.FormShow(Sender: TObject);
begin
  FormPosLoad(self);
end;


procedure TdlgList.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FormPosSave(self);
end;

end.

 